@@ -38,9 +38,10 @@ else: #pragma: no cover
 	import simplejson as json # extra dependency
 
 
-from zim.signals import SignalEmitter, ConnectorMixin
+from zim.signals import SignalEmitter, ConnectorMixin, SIGNAL_NORMAL
 from zim.utils import OrderedDict, FunctionThread
 from zim.fs import File, FileNotFoundError
+from zim.newfs import FileNotFoundError as NewFileNotFoundError
 from zim.errors import Error
 
 from .basedirs import XDG_CONFIG_HOME
@@ -49,17 +50,27 @@ from .basedirs import XDG_CONFIG_HOME
 logger = logging.getLogger('zim.config')
 
 
+class _MyMeta(SignalEmitter.__metaclass__, OrderedDict.__metaclass__):
+	# Combine meta classes to resolve conflict
+	pass
+
+
 class ControlledDict(OrderedDict, SignalEmitter, ConnectorMixin):
 	'''Sub-class of C{OrderedDict} that tracks modified state.
 	This modified state is recursive for nested C{ControlledDict}s.
 
-	Used as base class for L{SectionedConfigDict}, L{ConfigDict}
-	and L{HeadersDict}.
+	Used as base class for L{SectionedConfigDict} and L{ConfigDict}.
 
 	@signal: C{changed ()}: emitted when content of this dict changed,
 	or a nested C{ControlledDict} changed
 	'''
 
+	__metaclass__ = _MyMeta
+
+	__signals__ = {
+		'changed': (SIGNAL_NORMAL, None, ())
+	}
+
 	def __init__(self, E=None, **F):
 		OrderedDict.__init__(self, E, **F)
 		self._modified = False
@@ -80,7 +91,7 @@ class ControlledDict(OrderedDict, SignalEmitter, ConnectorMixin):
 
 	def update(self, E=(), **F):
 		# Only emit changed once here
-		with self.blocked_signals('changed'):
+		with self.block_signals('changed'):
 			OrderedDict.update(self, E, **F)
 		self.emit('changed')
 
@@ -120,12 +131,11 @@ class ConfigDefinition(object):
 	__slots__ = ('default', 'allow_empty')
 
 	def __init__(self, default, allow_empty=False):
-		self.default = default
 		if default is None:
 			allow_empty = True
 		self.allow_empty = allow_empty
 
-		self.check(default) # ensure that default follows check
+		self.default = self.check(default) # ensure that default follows check
 
 	def __eq__(self, other):
 		return self.__class__ == other.__class__ \
@@ -139,7 +149,7 @@ class ConfigDefinition(object):
 			if self.allow_empty:
 				return True
 			else:
-				raise ValueError, 'Value not allowed to be empty'
+				raise ValueError('Value not allowed to be empty')
 		else:
 			return False
 
@@ -180,7 +190,7 @@ class ConfigDefinitionByClass(ConfigDefinition):
 	'''
 	# TODO fully get rid of this class and replace by specialized classes
 
-	__slots__= ('klass',)
+	__slots__ = ('klass',)
 
 	def __init__(self, default, klass=None, allow_empty=False):
 		if klass is None:
@@ -215,15 +225,15 @@ class ConfigDefinitionByClass(ConfigDefinition):
 				return self.klass.new_from_zim_config(value)
 			except:
 				logger.debug('Error while converting %s to %s', value, self.klass, exc_info=1)
-				raise ValueError, 'Can not convert %s to %s' % (value, self.klass)
+				raise ValueError('Can not convert %s to %s' % (value, self.klass))
 		else:
-			raise ValueError, 'Value should be of type: %s' % self.klass.__name__
+			raise ValueError('Value should be of type: %s' % self.klass.__name__)
 
 	def tostring(self, value):
 		if hasattr(value, 'serialize_zim_config'):
 			return value.serialize_zim_config()
 		else:
-			return json.dumps(value, separators=(',',':'))
+			return json.dumps(value, separators=(',', ':'))
 				# specify separators for compact encoding
 
 
@@ -238,7 +248,7 @@ class Boolean(ConfigDefinition):
 		elif value in ('True', 'true', 'False', 'false'):
 			return value in ('True', 'true')
 		else:
-			raise ValueError, 'Must be True or False'
+			raise ValueError('Must be True or False')
 
 
 class String(ConfigDefinition):
@@ -259,7 +269,7 @@ class String(ConfigDefinition):
 		elif hasattr(value, 'serialize_zim_config'):
 			return value.serialize_zim_config()
 		else:
-			raise ValueError, 'Must be string'
+			raise ValueError('Must be string')
 
 	def tostring(self, value):
 		if value is None:
@@ -289,7 +299,7 @@ class Integer(ConfigDefinition):
 			try:
 				return int(value)
 			except:
-				raise ValueError, 'Must be integer'
+				raise ValueError('Must be integer')
 
 
 class Float(ConfigDefinition):
@@ -304,7 +314,7 @@ class Float(ConfigDefinition):
 			try:
 				return float(value)
 			except:
-				raise ValueError, 'Must be integer'
+				raise ValueError('Must be integer')
 
 
 class Choice(ConfigDefinition):
@@ -353,7 +363,7 @@ class Choice(ConfigDefinition):
 			elif isinstance(value, basestring) and value.lower() in choices:
 				return value.lower()
 			else:
-				raise ValueError, 'Value should be one of %s' % unicode(choices)
+				raise ValueError('Value should be one of %s' % unicode(choices))
 
 
 class Range(Integer):
@@ -379,7 +389,7 @@ class Range(Integer):
 		elif self.min <= value <= self.max:
 			return value
 		else:
-			raise ValueError, 'Value should be between %i and %i' % (self.min, self.max)
+			raise ValueError('Value should be between %i and %i' % (self.min, self.max))
 
 
 class Coordinate(ConfigDefinition):
@@ -391,7 +401,7 @@ class Coordinate(ConfigDefinition):
 
 	def __init__(self, default, allow_empty=False):
 		if default == (None, None):
-			allow_empty=True
+			allow_empty = True
 		ConfigDefinition.__init__(self, default, allow_empty)
 
 	def check(self, value):
@@ -413,7 +423,7 @@ class Coordinate(ConfigDefinition):
 			):
 				return value
 			else:
-				raise ValueError, 'Value should be a coordinate (tuple of 2 integers)'
+				raise ValueError('Value should be a coordinate (tuple of 2 integers)')
 
 value_is_coord = Coordinate # XXX for backward compatibility
 
@@ -433,11 +443,11 @@ def build_config_definition(default=None, check=None, allow_empty=False):
 	based on a default value an/or a check.
 	'''
 	if default is None and check is None:
-		raise AssertionError, 'At least provide either a default or a check'
+		raise AssertionError('At least provide either a default or a check')
 	elif check is None:
 		check = default.__class__
 
-	if isinstance(check, (type, types.ClassType)): # is a class
+	if isinstance(check, (type, type)): # is a class
 		if issubclass(check, ConfigDefinition):
 			return check(default, allow_empty=allow_empty)
 		elif check in _definition_classes:
@@ -453,7 +463,7 @@ def build_config_definition(default=None, check=None, allow_empty=False):
 			and isinstance(check[1], int)
 		return Range(default, check[0], check[1])
 	else:
-		raise ValueError, 'Unrecognized check type'
+		raise ValueError('Unrecognized check type')
 
 
 
@@ -480,10 +490,9 @@ class ConfigDict(ControlledDict):
 		assert not (E and F)
 		ControlledDict.__init__(self)
 		self.definitions = OrderedDict()
-		if E:
-			self._input = dict(E)
-		else:
-			self._input = F
+		self._input = {}
+		if E or F:
+			self.input(E or F)
 
 	def copy(self):
 		'''Shallow copy of the items
@@ -492,6 +501,7 @@ class ConfigDict(ControlledDict):
 		new = self.__class__()
 		new.update(self)
 		new._input.update(self._input)
+		new._keys[:] = list(self._keys)
 		return new
 
 	def update(self, E=None, **F):
@@ -513,10 +523,17 @@ class ConfigDict(ControlledDict):
 		keys in the dict, but want to preserve all of them when
 		writing back to a file.
 		'''
-		# TODO make this output ordered as well...
-		dump = dict(self._input)
-		dump.update(self)
-		return dump
+		return dict(self.all_items()) # FIXME should be OrderedDict, but causes test errors
+
+	def all_items(self):
+		# Like items() but returns both defined values and input values
+		for k in self._keys:
+			if k in self._values:
+				yield k, self._values[k]
+			elif k in self._input:
+				yield k, self._input[k]
+			else:
+				pass
 
 	# Note that OrderedDict optimizes __getitem__, cannot overload it
 
@@ -524,13 +541,23 @@ class ConfigDict(ControlledDict):
 		if k in self.definitions:
 			try:
 				v = self.definitions[k].check(v)
-			except ValueError, error:
-				raise ValueError, 'Invalid config value for %s: "%s" - %s' % (k, v, error.args[0])
+			except ValueError as error:
+				raise ValueError('Invalid config value for %s: "%s" - %s' % (k, v, error.args[0]))
 			else:
 				ControlledDict.__setitem__(self, k, v)
 		else:
 			raise KeyError('Config key "%s" has not been defined' % k)
 
+	def __delitem__(self, k):
+		if k in self._values:
+			ControlledDict.__delitem__(self, k)
+		else:
+			del self._input[k]
+			try:
+				self._keys.remove(k)
+			except ValueError:
+				pass
+
 	def input(self, E=None, **F):
 		'''Like C{update()} but won't raise on failures.
 		Values for undefined keys are stored and validated once the
@@ -539,7 +566,7 @@ class ConfigDict(ControlledDict):
 		'''
 		assert not (E and F)
 		update = E or F
-		if isinstance(update, collections.Mapping):
+		if hasattr(update, 'items'):
 			items = update.items()
 		else:
 			items = update
@@ -549,6 +576,8 @@ class ConfigDict(ControlledDict):
 				self._set_input(key, value)
 			else:
 				self._input[key] = value # validated later
+				if key not in self._keys:
+					self._keys.append(key)
 
 	def define(self, E=None, **F):
 		'''Set one or more defintions for this config dict
@@ -565,9 +594,8 @@ class ConfigDict(ControlledDict):
 		for key, definition in items:
 			if key in self.definitions:
 				if definition != self.definitions[key]:
-					raise AssertionError, \
-						'Key is already defined with different definition: %s\n%s != %s' \
-						% (key, definition, self.definitions[key])
+					raise AssertionError('Key is already defined with different definition: %s\n%s != %s'
+						% (key, definition, self.definitions[key]))
 				else:
 					continue
 
@@ -576,20 +604,20 @@ class ConfigDict(ControlledDict):
 				value = self._input.pop(key)
 				self._set_input(key, value)
 			else:
-				with self.blocked_signals('changed'):
+				with self.block_signals('changed'):
 					OrderedDict.__setitem__(self, key, definition.default)
 
 	def _set_input(self, key, value):
 		try:
 			value = self.definitions[key].check(value)
-		except ValueError, error:
+		except ValueError as error:
 			logger.warn(
 				'Invalid config value for %s: "%s" - %s',
 					key, value, error.args[0]
 			)
 			value = self.definitions[key].default
 
-		with self.blocked_signals('changed'):
+		with self.block_signals('changed'):
 			OrderedDict.__setitem__(self, key, value)
 
 	def setdefault(self, key, default, check=None, allow_empty=False):
@@ -683,7 +711,7 @@ class SectionedConfigDict(ControlledDict):
 		try:
 			return ControlledDict.__getitem__(self, k)
 		except KeyError:
-			with self.blocked_signals('changed'):
+			with self.block_signals('changed'):
 				ControlledDict.__setitem__(self, k, ConfigDict())
 			return ControlledDict.__getitem__(self, k)
 
@@ -734,10 +762,10 @@ class INIConfigFile(SectionedConfigDict):
 		SectionedConfigDict.__init__(self)
 		self.file = file
 		try:
-			with self.blocked_signals('changed'):
+			with self.block_signals('changed'):
 				self.read()
 			self.set_modified(False)
-		except FileNotFoundError:
+		except (FileNotFoundError, NewFileNotFoundError):
 			pass
 
 		if monitor:
@@ -746,14 +774,14 @@ class INIConfigFile(SectionedConfigDict):
 	def on_file_changed(self, *a):
 		if self.file.check_has_changed_on_disk():
 			try:
-				with self.blocked_signals('changed'):
+				with self.block_signals('changed'):
 					self.read()
-			except FileNotFoundError:
+			except (FileNotFoundError, NewFileNotFoundError):
 				pass
 			else:
 				# First emit top level to allow general changes
 				self.emit('changed')
-				with self.blocked_signals('changed'):
+				with self.block_signals('changed'):
 					for section in self.values():
 						section.emit('changed')
 				self.set_modified(False)
@@ -826,17 +854,20 @@ class INIConfigFile(SectionedConfigDict):
 		'''
 		lines = []
 		def dump_section(name, section):
-			try:
-				lines.append('[%s]\n' % name)
-				for key, value in section.items():
-					if not key.startswith('_'):
-						lines.append('%s=%s\n' % (key, section.definitions[key].tostring(value)))
-				lines.append('\n')
-			except:
-				logger.exception('Dumping section [%s] failed:\n%r', name, section)
+			lines.append('[%s]\n' % name)
+			for key, value in section.all_items():
+				if not key.startswith('_'):
+					try:
+						if key in section.definitions:
+							lines.append('%s=%s\n' % (key, section.definitions[key].tostring(value)))
+						else:
+							lines.append('%s=%s\n' % (key, value))
+					except:
+						logger.exception('Error serializing "%s" in section "[%s]"', key, name)
+			lines.append('\n')
 
 		for name, section in self.items():
-			if section and not name.startswith('_'):
+			if not name.startswith('_'):
 				if isinstance(section, list):
 					for s in section:
 						dump_section(name, s)
@@ -846,112 +877,6 @@ class INIConfigFile(SectionedConfigDict):
 		return lines
 
 
-class HeaderParsingError(Error):
-	'''Error when parsing a L{HeadersDict}'''
-
-	description = '''\
-Invalid data was found in a block with headers.
-This probably means the header block was corrupted
-and can not be read correctly.'''
-
-	def __init__(self, line):
-		self.msg = 'Invalid header >>%s<<' % line.strip('\n')
-
-
-class HeadersDict(ControlledDict):
-	'''This class maps a set of headers in the rfc822 format.
-	Can e.g. look like::
-
-		Content-Type: text/x-zim-wiki
-		Wiki-Format: zim 0.4
-		Creation-Date: 2010-12-14T14:15:09.134955
-
-	Header names are always kept in "title()" format to ensure
-	case-insensitivity.
-	'''
-
-	_is_header_re = re.compile('^([\w\-]+):\s+(.*)')
-	_is_continue_re = re.compile('^(\s+)(?=\S)')
-
-	def __init__(self, text=None):
-		'''Constructor
-
-		@param text: the header text, passed on to L{parse()}
-		'''
-		OrderedDict.__init__(self)
-		if not text is None:
-			self.parse(text)
-
-	def __getitem__(self, k):
-		return OrderedDict.__getitem__(self, k.title())
-
-	def __setitem__(self, k, v):
-		return OrderedDict.__setitem__(self, k.title(), v)
-
-	def read(self, lines):
-		'''Checks for headers at the start of the list of lines and
-		read them into the dict until the first empty line. Will remove
-		any lines belonging to the header block from the original list,
-		so after this method returns the input does no longer contain
-		the header block.
-		@param lines: a list of lines
-		'''
-		self._parse(lines, fatal=False)
-		if lines and lines[0].isspace():
-			lines.pop(0)
-
-	def parse(self, text):
-		'''Adds headers defined in 'text' to the dict.
-		Trailing whitespace is ignored.
-
-		@param text: a header block, either as string or as a list of lines.
-
-		@raises HeaderParsingError: when C{text} is not a valid header
-		block
-		'''
-		if isinstance(text, basestring):
-			lines = text.rstrip().splitlines(True)
-		else:
-			lines = text[:] # make copy so we do not destry the original
-		self._parse(lines)
-
-	def _parse(self, lines, fatal=True):
-		header = None
-		while lines:
-			is_header = self._is_header_re.match(lines[0])
-			if is_header:
-				header = is_header.group(1)
-				value  = is_header.group(2)
-				self[header] = value.strip()
-			elif self._is_continue_re.match(lines[0]) and not header is None:
-				self[header] += '\n' + lines[0].strip()
-			else:
-				if fatal:
-					raise HeaderParsingError, lines[0]
-				else:
-					break
-			lines.pop(0)
-
-	def dump(self, strict=False):
-		'''Serialize the dict to a header block in rfc822 header format.
-
-		@param strict: if C{True} lines will be properly terminated
-		with '\\r\\n' instead of '\\n'.
-
-		@returns: the header block as a list of lines
-		'''
-		buffer = []
-		for k, v in self.items():
-			v = v.strip().replace('\n', '\n\t')
-			buffer.extend((k, ': ', v, '\n'))
-		text = ''.join(buffer)
-
-		if strict:
-			text = text.replace('\n', '\r\n')
-
-		return text.splitlines(True)
-
-
 class HierarchicDict(object):
 	'''This class implements a data store that behaves as a hierarchig
 	dict of dicts. Each key in this object is considered a hierarchic
