@@ -26,7 +26,7 @@ from zim.plugins import PluginClass, extends, WindowExtension
 from zim.utils import WeakSet
 from zim.objectmanager import ObjectManager, CustomObjectClass
 from zim.config import String
-from zim.main import get_zim_application
+from zim.main import ZIM_APPLICATION
 from zim.gui.widgets import Dialog, ScrolledWindow, IconButton, InputEntry
 from zim.gui.objectmanager import CustomObjectWidget
 from zim.formats import ElementTreeModule as ElementTree
@@ -58,7 +58,7 @@ SYNTAX_WIKI_PANGO2 = [
 COLUMNS_ALIGNMENTS = {'left': ['left', gtk.STOCK_JUSTIFY_LEFT, _('Left')],  # T: alignment option
 					  'center': ['center', gtk.STOCK_JUSTIFY_CENTER, _('Center')],  # T: alignment option
 					  'right': ['right', gtk.STOCK_JUSTIFY_RIGHT, _('Right')],  # T: alignment option
-					  'normal': ['normal', None, _('Unspecified')],}  # T: alignment option
+					  'normal': ['normal', None, _('Unspecified')], }  # T: alignment option
 
 
 def reg_replace(string):
@@ -122,6 +122,10 @@ Exporting them to various formats (i.e. HTML/LaTeX) completes the feature set.
 		ObjectManager.register_object(OBJECT_TYPE, self.create_table)
 		self.connectto(self.preferences, 'changed', self.on_preferences_changed)
 
+	def teardown(self):
+		''' Deconstructor '''
+		ObjectManager.unregister_object(OBJECT_TYPE)
+
 	def create_table(self, attrib, text):
 		'''
 		Automatic way for displaying the table-object as a table within the wiki,
@@ -227,11 +231,6 @@ class MainWindowExtension(WindowExtension):
 		if self.window.ui.page:
 			self.window.ui.reload_page()
 
-	def teardown(self):
-		''' Deconstructor '''
-		ObjectManager.unregister_object(OBJECT_TYPE)
-		self.window.ui.reload_page()
-
 	@action(_('Table'), stock='zim-insert-table', readonly=False)  # T: menu item
 	def insert_table(self):
 		'''Run the EditTableDialog'''
@@ -276,7 +275,7 @@ class TableViewObject(CustomObjectClass):
 			if isinstance(v, list):
 				v = ','.join(map(str, v))
 			_attrib[k] = v
-		CustomObjectClass.__init__(self, _attrib, [header]+rows)
+		CustomObjectClass.__init__(self, _attrib, [header] + rows)
 		self.attrib = {'type': OBJECT_TYPE} # just to be sure
 
 		self._tableattrib = attrib
@@ -308,7 +307,7 @@ class TableViewObject(CustomObjectClass):
 
 	def _get_liststore(self, reset=False):
 		if reset or not self._liststore:
-			cols = [str]*len(self._header)
+			cols = [str] * len(self._header)
 			self._liststore = gtk.ListStore(*cols)
 			for trow in self._rows:
 				self._liststore.append(trow)
@@ -374,10 +373,12 @@ class TableViewObject(CustomObjectClass):
 		# update data
 		if self._liststore:
 			liststore = self._get_liststore()
+			self._header = headers
 			self._rows = self._update_rows(liststore, id_mapping, len(headers))
 			liststore = self._get_liststore(reset=True)
 		else:
 			liststore = None
+			self._header = headers
 			self._rows = self._update_rows(self._rows, id_mapping, len(headers))
 
 		self.set_aligns(aligns)
@@ -396,7 +397,7 @@ class TableViewObject(CustomObjectClass):
 		''' Old value of cells are used in the new table, but only if its column is not deleted '''
 		new_rows = []
 		for oldrow in old_rows:
-				newrow = [' ']*nr_cols
+				newrow = [' '] * nr_cols
 				for v, k in id_mapping.iteritems():
 					newrow[v] = oldrow[k]
 				new_rows.append(newrow)
@@ -617,7 +618,7 @@ class TableViewWidget(CustomObjectWidget):
 				tooltips.set_tip(button, tooltip)
 				toolbar.insert(button, pos)
 
-		toolbar.set_size_request(300,-1)
+		toolbar.set_size_request(300, -1)
 		toolbar.set_icon_size(gtk.ICON_SIZE_MENU)
 
 		return toolbar
@@ -644,6 +645,9 @@ class TableViewWidget(CustomObjectWidget):
 		'''
 		treeview = gtk.TreeView(liststore)
 
+		# Set default sorting function.
+		liststore.set_default_sort_func(lambda *a: 0)
+
 		for i, headcol in enumerate(headers):
 			cell = gtk.CellRendererText()
 			tview_column = gtk.TreeViewColumn(headcol, cell)
@@ -688,7 +692,7 @@ class TableViewWidget(CustomObjectWidget):
 		col_widget.show()
 
 
-		col_label = gtk.Label('<u>'+title+'</u>')
+		col_label = gtk.Label('<u>' + title + '</u>')
 		col_label.set_use_markup(True)
 		col_label.show()
 		col_widget.pack_start(col_label)
@@ -734,12 +738,12 @@ class TableViewWidget(CustomObjectWidget):
 		Displays a context-menu on right button click
 		Opens the link of a tablecell on CTRL pressed and left button click
 		'''
-		if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1 and event.get_state() &  gtk.gdk.CONTROL_MASK:
+		if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1 and event.get_state() & gtk.gdk.CONTROL_MASK:
 			# With CTRL + LEFT-Mouse-Click link of cell is opened
 			cellvalue = self.fetch_cell_by_event(event, treeview)
 			linkvalue = self.get_linkurl(cellvalue)
 			if linkvalue:
-				self.obj.emit('link-clicked', {'href': linkvalue})
+				self.emit('link-clicked', {'href': unicode(linkvalue)})
 			return
 
 		if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
@@ -789,7 +793,10 @@ class TableViewWidget(CustomObjectWidget):
 			self.selection_info()
 			return
 
-		row = len(self.treeview.get_columns())*['']
+		# Set default sorting.
+		model.set_sort_column_id(-1, gtk.SORT_ASCENDING)
+
+		row = len(self.treeview.get_columns()) * ['']
 		path = model.insert_after(treeiter, row)
 		self.obj.set_modified(True)
 
@@ -838,18 +845,26 @@ class TableViewWidget(CustomObjectWidget):
 			return
 		newiter = model.get_iter((newpos,))
 
-		model.swap(treeiter, newiter)
+		# Set default sorting.
+		model.set_sort_column_id(-1, gtk.SORT_ASCENDING)
+
+		# Change values of two rows.
+		for col in range(model.get_n_columns()):
+			value = model.get_value(treeiter, col)
+			newvalue = model.get_value(newiter, col)
+			model.set_value(newiter, col, value)
+			model.set_value(treeiter, col, newvalue)
+
 		self.obj.set_modified(True)
 
 
 	def on_open_link(self, action, link):
 		''' Context menu: Open a link, which is written in a cell '''
-		self.obj.emit('link-clicked', {'href': link})
+		self.emit('link-clicked', {'href': unicode(link)})
 
 	def on_open_help(self, action):
 		''' Context menu: Open help '''
-		get_zim_application('--manual', 'Plugins:Table Editor').spawn()
-		pass
+		ZIM_APPLICATION.run('--manual', 'Plugins:Table Editor')
 
 	def on_change_columns(self, action):
 		''' Context menu: Edit table, run the EditTableDialog '''
@@ -1126,7 +1141,7 @@ class EditTableDialog(Dialog):
 	def do_response_ok(self):
 		''' Dialog Window is closed with "OK" '''
 		self.autosave_title_cell()
-		self.result = [[m[0], m[1], m[2],m[3]] for m in self.treeview.get_model()]
+		self.result = [[m[0], m[1], m[2], m[3]] for m in self.treeview.get_model()]
 		return True
 
 	def do_response_cancel(self):
@@ -1177,7 +1192,7 @@ class EditTableDialog(Dialog):
 		self.autosave_title_cell()
 		(model, treeiter) = self.treeview.get_selection().get_selected()
 		if not treeiter:  # preselect first entry
-			path = model.iter_n_children(None)-1
+			path = model.iter_n_children(None) - 1
 			treeiter = model.get_iter(path)
 		newiter = model.insert_after(treeiter, self.default_column_item)
 		self.treeview.set_cursor_on_cell(model.get_path(newiter), self.treeview.get_column(0), start_editing=True)
